@@ -7,7 +7,7 @@ use test::Bencher;
 use rand::seq::SliceRandom;
 
 use keeshond::gameloop::{GameInfo, GameControl};
-use keeshond::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, ThinkerArgs, EntityComponent};
+use keeshond::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, ThinkerArgs, EntityComponent, ComponentRef, ComponentRefMut};
 
 const NUM_ITER : usize = 10_000;
 
@@ -35,6 +35,26 @@ impl Component for Position
     fn capacity_hint() -> usize where Self : Sized { NUM_ITER }
 }
 
+impl<'a> ComponentRef<'a> for Position
+{
+    type StoreRef = &'a Position;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for Position
+{
+    type StoreRefMut = &'a mut Position;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 struct Velocity
 {
     vel_x : f64,
@@ -59,6 +79,26 @@ impl Component for Velocity
     fn capacity_hint() -> usize where Self : Sized { NUM_ITER }
 }
 
+impl<'a> ComponentRef<'a> for Velocity
+{
+    type StoreRef = &'a Velocity;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for Velocity
+{
+    type StoreRefMut = &'a mut Velocity;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 struct PositionVelocity
 {
     x : f64,
@@ -87,6 +127,26 @@ impl Component for PositionVelocity
     fn capacity_hint() -> usize where Self : Sized { NUM_ITER }
 }
 
+impl<'a> ComponentRef<'a> for PositionVelocity
+{
+    type StoreRef = &'a PositionVelocity;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for PositionVelocity
+{
+    type StoreRefMut = &'a mut PositionVelocity;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 struct PositionOrdered
 {
     x : f64,
@@ -112,6 +172,26 @@ impl Component for PositionOrdered
     fn capacity_hint() -> usize where Self : Sized { NUM_ITER }
 }
 
+impl<'a> ComponentRef<'a> for PositionOrdered
+{
+    type StoreRef = &'a PositionOrdered;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for PositionOrdered
+{
+    type StoreRefMut = &'a mut PositionOrdered;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 struct VelocityOrdered
 {
     vel_x : f64,
@@ -136,6 +216,26 @@ impl Component for VelocityOrdered
     fn capacity_hint() -> usize where Self : Sized { NUM_ITER }
 }
 
+impl<'a> ComponentRef<'a> for VelocityOrdered
+{
+    type StoreRef = &'a VelocityOrdered;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for VelocityOrdered
+{
+    type StoreRefMut = &'a mut VelocityOrdered;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 
 ////////////////////////////////////////
 
