@@ -32,7 +32,8 @@ use crate::renderer::draw_control_private::RendererNewInfo;
 const MILLISECONDS_PER_NANOSECOND : f64 = 0.000001;
 const BUSY_WAIT_MARGIN : u64 = 2; // For smooth results on Windows. Linux can probably get away with 1.
 const AUTOMATIC_LERP_CAP_MULTIPLIER : f64 = 1.05;
-const MAX_POST_SLOWDOWN_CATCHUP : u32 = 10;
+const MAX_HITCH_MS : u64 = 100;
+const MAX_POST_SLOWDOWN_CATCHUP : u32 = 6;
 const MAX_FRAME_JIFFY_SUBTRACT : u32 = 1000;
 const MAX_FPS_STAT_CATCHUP : u64 = 3;
 const LOW_ENERGY_TIMER_MIN : u32 = 60;
@@ -342,6 +343,7 @@ pub struct GameloopStopwatch
     jiffies : Instant,
     jiffies_this_frame : Duration,
     jiffies_this_second : Duration,
+    jiffies_last_frame : Duration,
     tics_this_frame : u32,
     tics_this_second : u32,
     frames_this_second : u32,
@@ -357,6 +359,7 @@ impl GameloopStopwatch
             jiffies : Instant::now(),
             jiffies_this_frame : Duration::new(0, 0),
             jiffies_this_second : Duration::new(0, 0),
+            jiffies_last_frame : Duration::new(0, 0),
             tics_this_frame : 0,
             tics_this_second : 0,
             frames_this_second : 0,
@@ -374,6 +377,7 @@ impl GameloopStopwatch
         self.in_frame = true;
         
         self.jiffies = Instant::now();
+        self.jiffies_last_frame = self.jiffies_this_frame;
         self.jiffies_this_frame = Duration::new(0, 0);
         self.tics_this_frame = 0;
     }
@@ -1164,8 +1168,19 @@ impl Gameloop
             self.time.thought_yet = true;
             self.control.low_energy_timer += 1;
 
+            let max_think_jiffies = self.time.target_frametime;
+
             // Prevent overflow if CPU cannot process all the frames
-            if (self.time.think_timer.jiffies_this_frame()) > self.time.target_frametime
+            //  Scenario 1: Hitch for a single frame. Could be OS taking resource time away from
+            //              the program for a brief moment, or a long initialization process. For
+            //              this we should allow for a single shot of several frames worth of lag.
+            //  Scenario 2: Continuous slow processing. Game is simply doing too much work over
+            //              multiple frames. If previous think took longer than a frame and this
+            //              think is also taking longer than a frame, stop processing for smoother
+            //              slowdown should it occur.
+            if self.time.think_timer.jiffies_this_frame() > max_think_jiffies + Duration::from_millis(MAX_HITCH_MS)
+                || (self.time.think_timer.jiffies_this_frame() > max_think_jiffies
+                && self.time.think_timer.jiffies_last_frame > max_think_jiffies)
             {
                 while self.time.jiffy_queue > self.time.target_frametime
                 {
@@ -1220,7 +1235,8 @@ impl Gameloop
         {
             let mut interpolation_amount = 0.0;
 
-            if self.control.frame_interpolation_active() && !self.low_energy_timer_expired()
+            if self.control.frame_interpolation_active() && self.time.lag_this_frame == 0
+                && !self.low_energy_timer_expired()
             {
                 let jiffy_offset = self.time.jiffy_queue.as_nanos() as f64;
                 let jiffy_target = self.time.target_frametime.as_nanos() as f64;
