@@ -65,7 +65,7 @@ extern crate typenum;
 extern crate generic_array;
 extern crate bitflags;
 extern crate cgmath;
-#[cfg(feature = "renderer")] #[macro_use] extern crate glium;
+#[macro_use] extern crate glium;
 #[cfg(windows)] extern crate winapi;
 
 mod thirdparty;
