@@ -16,8 +16,8 @@ use crate::datapack::{DataId, DataStore, PreparedStore, PreparedStoreError, Data
 use crate::renderer::{DrawControl, DrawControlResult, DrawTransform, DrawSpriteRawInfo, DrawBackgroundColorInfo, RendererError, Sheet, Shader, ViewportMode, DrawSpriteInfo, Color};
 use crate::renderer::draw_control_private::{DrawControlBackend, RendererNewInfo};
 
-#[cfg(feature = "imgui")] use imgui_glium_renderer;
-#[cfg(feature = "imgui")] use glium::uniforms::SamplerBehavior;
+#[cfg(feature = "imgui_feature")] use imgui_glium_renderer;
+#[cfg(feature = "imgui_feature")] use glium::uniforms::SamplerBehavior;
 
 const INSTANCES_MAX : usize = 2048; // How many instances to store in each instance buffer.
 const INSTANCE_BUFFER_MAX: usize = 4; // How many instance buffers to have.
@@ -101,7 +101,7 @@ struct GpuSheetPreparer
     #[allow(dead_code)]
     default_texture : Rc<Texture2d>,
     texture_filtering : bool,
-    #[cfg(feature = "imgui")]
+    #[cfg(feature = "imgui_feature")]
     imgui_renderer : Rc<RefCell<Option<imgui_glium_renderer::Renderer>>>
 }
 
@@ -187,7 +187,7 @@ impl GpuSheetPreparer
         }
     }
 
-    #[cfg(feature = "imgui")]
+    #[cfg(feature = "imgui_feature")]
     fn update_imgui_texture(&mut self, id : DataId<Sheet>, handle : &mut GpuSheetHandle)
     {
         let mut imgui_borrow = self.imgui_renderer.borrow_mut();
@@ -246,7 +246,7 @@ impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
 
         data.texture_dirty = false;
 
-        #[cfg(feature = "imgui")]
+        #[cfg(feature = "imgui_feature")]
         self.update_imgui_texture(id, &mut handle);
 
         handle
@@ -255,7 +255,7 @@ impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
     #[allow(unused_variables)]
     fn unprepare(&mut self, _prepared : &mut GpuSheetHandle, id : DataId<Sheet>)
     {
-        #[cfg(feature = "imgui")]
+        #[cfg(feature = "imgui_feature")]
         {
             let mut imgui_borrow = self.imgui_renderer.borrow_mut();
 
@@ -288,7 +288,7 @@ impl DataPreparer<Sheet, GpuSheetHandle> for GpuSheetPreparer
             *prepared = self.make_sheet_handle_from_texture(data, prepared.texture.clone());
         }
 
-        #[cfg(feature = "imgui")]
+        #[cfg(feature = "imgui_feature")]
         self.update_imgui_texture(id, prepared);
 
         data.texture_dirty = false;
@@ -413,7 +413,7 @@ pub(crate) struct GlDrawControl
     last_view_transform : DrawTransform,
     gpu_sheet_store : PreparedStore<Sheet, GpuSheetHandle>,
     gpu_shader_store : PreparedStore<Shader, GpuShaderHandle>,
-    #[cfg(feature = "imgui")]
+    #[cfg(feature = "imgui_feature")]
     imgui_renderer : Rc<RefCell<Option<imgui_glium_renderer::Renderer>>>
 }
 
@@ -775,14 +775,14 @@ impl DrawControlBackend for GlDrawControl
         let window = display_raw.window_clone();
         
         let display = Rc::new(RefCell::new(display_raw));
-        #[cfg(feature = "imgui")]
+        #[cfg(feature = "imgui_feature")]
         let imgui_renderer = Rc::new(RefCell::new(None));
         let sheet_data_preparer = Box::new(GpuSheetPreparer
         {
             display : display.clone(),
             default_texture: default_texture.clone(),
             texture_filtering : renderer_new_info.texture_filtering,
-            #[cfg(feature = "imgui")]
+            #[cfg(feature = "imgui_feature")]
             imgui_renderer: imgui_renderer.clone()
         });
         let shader_data_preparer = Box::new(GpuShaderPreparer
@@ -828,7 +828,7 @@ impl DrawControlBackend for GlDrawControl
             last_view_transform : DrawTransform::identity(),
             gpu_sheet_store,
             gpu_shader_store,
-            #[cfg(feature = "imgui")]
+            #[cfg(feature = "imgui_feature")]
             imgui_renderer,
         }), window))
     }
@@ -843,7 +843,7 @@ impl DrawControlBackend for GlDrawControl
         self.gpu_shader_store.sync(shader_store)
     }
     
-    #[cfg(feature = "imgui")]
+    #[cfg(feature = "imgui_feature")]
     fn draw_imgui(&mut self, ui : imgui::Ui)
     {
         self.perform_draw_batch();
@@ -1015,7 +1015,7 @@ impl DrawControlBackend for GlDrawControl
         }
     }
     
-    #[cfg(feature = "imgui")]
+    #[cfg(feature = "imgui_feature")]
     fn init_imgui_renderer(&mut self, imgui : &mut imgui::Context) -> Result<(), RendererError>
     {
         let mut imgui_borrow = self.imgui_renderer.borrow_mut();
@@ -1046,7 +1046,7 @@ impl DrawControlBackend for GlDrawControl
         Ok(())
     }
 
-    #[cfg(feature = "imgui")]
+    #[cfg(feature = "imgui_feature")]
     fn update_imgui_renderer(&mut self, imgui : &mut imgui::Context)
     {
         let mut imgui_borrow = self.imgui_renderer.borrow_mut();
