@@ -1,5 +1,7 @@
 //! Keeshond's very own ECS framework, and where all the action in your game takes place
 
+pub mod iter;
+
 use bitflags::bitflags;
 use downcast_rs::Downcast;
 use rustc_hash::FxHasher;
@@ -23,14 +25,13 @@ use std::cell::{RefCell, RefMut, Ref, BorrowMutError, BorrowError};
 use std::collections::{HashMap, VecDeque};
 use std::cmp::Ordering;
 use std::fmt::{Display, Formatter};
-use std::iter::Iterator;
 use std::hash::BuildHasherDefault;
 use std::str::FromStr;
 
 use crate::datapack::DataMultistore;
 use crate::gameloop::GameControl;
 use crate::renderer::{DrawControl, DrawTransform};
-
+use crate::scene::iter::{ComponentAllIter, ComponentAllIterMut, ComponentFlagsIter, ComponentFilterIterMut, ComponentIter, ComponentIterMut, ComponentFilterIter, ComponentFlagsIterMut};
 
 #[cfg(feature = "imgui_feature")]
 use std::marker::PhantomData;
@@ -209,7 +210,7 @@ impl Default for Entity
 }
 
 /// A data struct which may be added to an [Entity]
-pub trait Component : Downcast
+pub trait Component
 {
     /// Whether to maintain insertion order when removing components. By default, the slower method
     /// that maintains ordering is used. The performance penalty is based on the size of the data
@@ -220,8 +221,31 @@ pub trait Component : Downcast
     
     /// How many components of this type that memory should be reserved for in advance.
     fn capacity_hint() -> usize where Self : Sized { 0 }
+
+    /// The priority used to determine which component types are added to the scene first when
+    ///  doing deferred spawns. Lower numbers are added first.
+    fn add_priority() -> i32 where Self : Sized { 0 }
+}
+
+pub trait ComponentRef<'a>
+{
+    /// Reference type to return when retrieving or iterating over the component via [ComponentStore]
+    type StoreRef;
+
+    /// Used to get a reference type for this object when retrieving or iterating over the
+    ///  component via [ComponentStore]
+    fn store_get(&'a self) -> Self::StoreRef;
+}
+
+pub trait ComponentRefMut<'a>
+{
+    /// Reference type to return when mutably retrieving or iterating over the component via [ComponentStore]
+    type StoreRefMut;
+
+    /// Used to get a mutable reference type for this object when retrieving or iterating over the
+    ///  component via [ComponentStore]
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut;
 }
-impl_downcast!(Component);
 
 // A struct that contains both an entity and a component, for passing entity-component pairs around
 pub struct EntityComponent<C : Component + 'static>
@@ -661,7 +685,7 @@ bitflags!
         /// Component is frozen because it is too far outside the view
         const REASON_CULLED = 0b0000_0000_0000_0100;
         /// Component is frozen because it is hidden (even though it may be inside the view)
-        const REASON_HIDDEN = 0b0000_0000_0000_0100;
+        const REASON_HIDDEN = 0b0000_0000_0000_1000;
         /// Component is frozen due to a custom behavior toggle (1)
         const REASON_BEHAVIOR_1 = 0b0000_0000_0001_0000;
         /// Component is frozen due to a custom behavior toggle (2)
@@ -775,28 +799,28 @@ impl<C : Component + 'static> ComponentStore<C>
     
     /// Returns an immutable reference to the [Component] for the given [Entity], or [None]
     /// if it does not exist.
-    pub fn get_entity(&self, entity : &Entity) -> Option<&C>
+    pub fn get_entity<'a>(&'a self, entity : &Entity) -> Option<C::StoreRef> where C : ComponentRef<'a>
     {
         match self.get_nth_index(entity)
         {
             None => { None }
             Some(index) =>
             {
-                Some(&self.component_list[index].component)
+                Some(self.component_list[index].component.store_get())
             }
         }
     }
     
     /// Returns a mutable reference to the [Component] for the given [Entity], or [None]
     /// if it does not exist.
-    pub fn get_entity_mut(&mut self, entity : &Entity) -> Option<&mut C>
+    pub fn get_entity_mut<'a>(&'a mut self, entity : &Entity) -> Option<C::StoreRefMut> where C : ComponentRefMut<'a>
     {
         match self.get_nth_index(entity)
         {
             None => { None }
             Some(index) =>
             {
-                Some(&mut self.component_list[index].component)
+                Some(self.component_list[index].component.store_get_mut())
             }
         }
     }
@@ -804,31 +828,14 @@ impl<C : Component + 'static> ComponentStore<C>
     /// Returns a clone of the [Component] for the given [Entity]. This is useful
     ///  for getting around issues with the Rust borrow checker preventing you from accessing more
     ///  than one component in the store at a time.
-    pub fn clone_from_entity(&self, entity : &Entity) -> Option<C> where C : Clone
-    {
-        if let Some(component) = self.get_entity(&entity)
-        {
-            return Some(component.clone());
-        }
-        
-        None
-    }
-
-    /// Sets the [Component] for the given [Entity] in the store. Useful to "commit" changes
-    ///  to a component that you cloned previously with [ComponentStore::clone_from_entity()].
-    ///  Returns true if there is a component for the given entity.
-    pub fn set_entity(&mut self, entity : &Entity, component : C) -> bool
+    pub fn clone_from_entity<'a>(&'a self, entity : &Entity) -> Option<C> where C : Clone + ComponentRef<'a>
     {
         match self.get_nth_index(entity)
         {
-            None => false,
+            None => { None }
             Some(index) =>
             {
-                let existing = &mut self.component_list[index];
-
-                existing.component = component;
-
-                return true;
+                Some(self.component_list[index].component.clone())
             }
         }
     }
@@ -837,14 +844,14 @@ impl<C : Component + 'static> ComponentStore<C>
     ///  useful for when you want to get the first, last, or random component, etc. The index
     ///  you use here should not be cached, as the components may move around in the store.
     ///  In most cases you should use [ComponentStore::get_entity()] instead.
-    pub fn get_nth(&self, index : usize) -> Option<(Entity, &C)>
+    pub fn get_nth<'a>(&'a self, index : usize) -> Option<(Entity, C::StoreRef)> where C : ComponentRef<'a>
     {
         match self.component_list.get(index)
         {
             None => { None }
             Some(entry) =>
             {
-                Some((entry.entity.clone(), &entry.component))
+                Some((entry.entity.clone(), entry.component.store_get()))
             }
         }
     }
@@ -853,24 +860,24 @@ impl<C : Component + 'static> ComponentStore<C>
     ///  useful for when you want to get the first, last, or random component, etc. The index
     ///  you use here should not be cached, as the components may move around in the store.
     ///  In most cases you should use [ComponentStore::get_entity_mut()] instead.
-    pub fn get_nth_mut(&mut self, index : usize) -> Option<(Entity, &mut C)>
+    pub fn get_nth_mut<'a>(&'a mut self, index : usize) -> Option<(Entity, C::StoreRefMut)> where C : ComponentRefMut<'a>
     {
         match self.component_list.get_mut(index)
         {
             None => { None }
             Some(entry) =>
             {
-                Some((entry.entity.clone(), &mut entry.component))
+                Some((entry.entity.clone(), entry.component.store_get_mut()))
             }
         }
     }
 
-    pub fn first(&self) -> Option<(Entity, &C)>
+    pub fn first<'a>(&'a self) -> Option<(Entity, C::StoreRef)> where C : ComponentRef<'a>
     {
         self.get_nth(0)
     }
 
-    pub fn first_mut(&mut self) -> Option<(Entity, &mut C)>
+    pub fn first_mut<'a>(&'a mut self) -> Option<(Entity, C::StoreRefMut)> where C : ComponentRefMut<'a>
     {
         self.get_nth_mut(0)
     }
@@ -884,7 +891,7 @@ impl<C : Component + 'static> ComponentStore<C>
     /// Returns true if there is a [Component] for the given [Entity]
     pub fn contains(&self, entity : &Entity) -> bool
     {
-        self.get_entity(entity).is_some()
+        self.get_nth_index(entity).is_some()
     }
 
     /// Returns the [FreezeFlags] for the given entity.
@@ -902,7 +909,7 @@ impl<C : Component + 'static> ComponentStore<C>
 
     /// Returns the [FreezeFlags] for the "n-th" [Component] in the store. The index
     ///  you use here should not be cached, as the components may move around in the store.
-    ///  In most cases you should use [ComponentStore::active()] instead.
+    ///  In most cases you should use [ComponentStore::freeze_flags()] instead.
     pub fn freeze_flags_nth(&self, index : usize) -> Option<FreezeFlags>
     {
         match self.component_list.get(index)
@@ -969,36 +976,22 @@ impl<C : Component + 'static> ComponentStore<C>
 
     /// Returns a [ComponentIter] that iterates through all [Components](Component)
     ///  in the store that are unfrozen.
-    pub fn iter(&self) -> ComponentIter<C>
+    pub fn iter<'a>(&'a self) -> ComponentIter<C> where C : ComponentRef<'a>
     {
         ComponentIter::<C>::new(self)
     }
 
     /// Returns a [ComponentIterMut] that iterates through all [Components](Component)
     ///  in the store that are unfrozen.
-    pub fn iter_mut(&mut self) -> ComponentIterMut<C>
+    pub fn iter_mut<'a>(&'a mut self) -> ComponentIterMut<C> where C : ComponentRefMut<'a>
     {
         ComponentIterMut::<C>::new(self)
     }
-    
-    /// Returns a [ComponentFilterIter] that iterates through all [Components](Component)
-    ///  in the store that are unfrozen according to the given [FreezeFlags].
-    pub fn iter_filtered(&self, freeze_flags : FreezeFlags) -> ComponentFilterIter<C>
-    {
-        ComponentFilterIter::<C>::new(self, freeze_flags)
-    }
-    
-    /// Returns a [ComponentFilterIterMut] that iterates through all [Components](Component)
-    ///  in the store that are unfrozen according to the given [FreezeFlags].
-    pub fn iter_filtered_mut(&mut self, freeze_flags : FreezeFlags) -> ComponentFilterIterMut<C>
-    {
-        ComponentFilterIterMut::<C>::new(self, freeze_flags)
-    }
 
     /// Returns a [ComponentAllIter] that iterates through all [Components](Component)
     ///  in the store. Unlike [ComponentStore::iter()] and [ComponentStore::iter_filtered()], this
     ///  includes frozen components as well.
-    pub fn iter_all(&self) -> ComponentAllIter<C>
+    pub fn iter_all<'a>(&'a self) -> ComponentAllIter<C> where C : ComponentRef<'a>
     {
         ComponentAllIter::<C>::new(self)
     }
@@ -1006,10 +999,38 @@ impl<C : Component + 'static> ComponentStore<C>
     /// Returns a [ComponentAllIterMut] that iterates through all [Components](Component)
     ///  in the store. Unlike [ComponentStore::iter_mut()] and
     ///  [ComponentStore::iter_filtered_mut()], this includes frozen components as well.
-    pub fn iter_all_mut(&mut self) -> ComponentAllIterMut<C>
+    pub fn iter_all_mut<'a>(&'a mut self) -> ComponentAllIterMut<C> where C : ComponentRefMut<'a>
     {
         ComponentAllIterMut::<C>::new(self)
     }
+
+    /// Returns a [ComponentFilterIter] that iterates through all [Components](Component)
+    ///  in the store that are unfrozen according to the given [FreezeFlags].
+    pub fn iter_filtered<'a>(&'a self, freeze_flags : FreezeFlags) -> ComponentFilterIter<C> where C : ComponentRef<'a>
+    {
+        ComponentFilterIter::<C>::new(self, freeze_flags)
+    }
+
+    /// Returns a [ComponentFilterIterMut] that iterates through all [Components](Component)
+    ///  in the store that are unfrozen according to the given [FreezeFlags].
+    pub fn iter_filtered_mut<'a>(&'a mut self, freeze_flags : FreezeFlags) -> ComponentFilterIterMut<C> where C : ComponentRefMut<'a>
+    {
+        ComponentFilterIterMut::<C>::new(self, freeze_flags)
+    }
+
+    /// Returns a [ComponentFilterIter] that iterates through all [Components](Component)
+    ///  in the store along with their [FreezeFlags] for custom filtering.
+    pub fn iter_flags<'a>(&'a self) -> ComponentFlagsIter<C> where C : ComponentRef<'a>
+    {
+        ComponentFlagsIter::<C>::new(self)
+    }
+
+    /// Returns a [ComponentFlagsIterMut] that iterates through all [Components](Component)
+    ///  in the store along with their [FreezeFlags] for custom filtering.
+    pub fn iter_flags_mut<'a>(&'a mut self) -> ComponentFlagsIterMut<C> where C : ComponentRefMut<'a>
+    {
+        ComponentFlagsIterMut::<C>::new(self)
+    }
     
     pub fn singleton(&self) -> Option<&C>
     {
@@ -1140,211 +1161,6 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
 ////////////////////////////////////////////////////////////
 
 
-pub struct ComponentAllIter<'a, C : Component + 'static>
-{
-    data_iter : std::slice::Iter<'a, ComponentListEntry<C>>,
-}
-
-impl<'a, C : Component + 'static> ComponentAllIter<'a, C>
-{
-    fn new(store : &ComponentStore<C>) -> ComponentAllIter<C>
-    {
-        ComponentAllIter::<C> { data_iter : store.component_list.iter() }
-    }
-}
-
-impl<'a, C : Component + 'static> Iterator for ComponentAllIter<'a, C>
-{
-    type Item = (Entity, &'a C);
-    
-    fn next(&mut self) -> Option<(Entity, &'a C)>
-    {
-        if let Some(entry) = self.data_iter.next()
-        {
-            return Some((entry.entity.clone(), &entry.component));
-        }
-        
-        None
-    }
-}
-
-
-pub struct ComponentAllIterMut<'a, C : Component + 'static>
-{
-    data_iter : std::slice::IterMut<'a, ComponentListEntry<C>>,
-}
-
-impl<'a, C : Component + 'static> ComponentAllIterMut<'a, C>
-{
-    fn new(store : &mut ComponentStore<C>) -> ComponentAllIterMut<C>
-    {
-        ComponentAllIterMut::<C> { data_iter : store.component_list.iter_mut() }
-    }
-}
-
-impl<'a, C : Component + 'static> Iterator for ComponentAllIterMut<'a, C>
-{
-    type Item = (Entity, &'a mut C);
-    
-    fn next(&mut self) -> Option<(Entity, &'a mut C)>
-    {
-        if let Some(entry) = self.data_iter.next()
-        {
-            return Some((entry.entity.clone(), &mut entry.component));
-        }
-        
-        None
-    }
-}
-
-
-pub struct ComponentIter<'a, C : Component + 'static>
-{
-    data_iter : std::slice::Iter<'a, ComponentListEntry<C>>
-}
-
-impl<'a, C : Component + 'static> ComponentIter<'a, C>
-{
-    fn new(store : &ComponentStore<C>) -> ComponentIter<C>
-    {
-        ComponentIter::<C>
-        {
-            data_iter : store.component_list.iter()
-        }
-    }
-}
-
-impl<'a, C : Component + 'static> Iterator for ComponentIter<'a, C>
-{
-    type Item = (Entity, &'a C);
-
-    fn next(&mut self) -> Option<(Entity, &'a C)>
-    {
-        while let Some(entry) = self.data_iter.next()
-        {
-            if entry.freeze_flags.is_empty()
-            {
-                return Some((entry.entity.clone(), &entry.component));
-            }
-        }
-
-        None
-    }
-}
-
-
-pub struct ComponentIterMut<'a, C : Component + 'static>
-{
-    data_iter : std::slice::IterMut<'a, ComponentListEntry<C>>
-}
-
-impl<'a, C : Component + 'static> ComponentIterMut<'a, C>
-{
-    fn new(store : &mut ComponentStore<C>) -> ComponentIterMut<C>
-    {
-        ComponentIterMut::<C>
-        {
-            data_iter : store.component_list.iter_mut()
-        }
-    }
-}
-
-impl<'a, C : Component + 'static> Iterator for ComponentIterMut<'a, C>
-{
-    type Item = (Entity, &'a mut C);
-
-    fn next(&mut self) -> Option<(Entity, &'a mut C)>
-    {
-        while let Some(entry) = self.data_iter.next()
-        {
-            if entry.freeze_flags.is_empty()
-            {
-                return Some((entry.entity.clone(), &mut entry.component));
-            }
-        }
-
-        None
-    }
-}
-
-
-pub struct ComponentFilterIter<'a, C : Component + 'static>
-{
-    data_iter : std::slice::Iter<'a, ComponentListEntry<C>>,
-    freeze_flags : FreezeFlags
-}
-
-impl<'a, C : Component + 'static> ComponentFilterIter<'a, C>
-{
-    fn new(store : &ComponentStore<C>, freeze_flags : FreezeFlags) -> ComponentFilterIter<C>
-    {
-        ComponentFilterIter::<C>
-        {
-            data_iter : store.component_list.iter(),
-            freeze_flags
-        }
-    }
-}
-
-impl<'a, C : Component + 'static> Iterator for ComponentFilterIter<'a, C>
-{
-    type Item = (Entity, &'a C);
-
-    fn next(&mut self) -> Option<(Entity, &'a C)>
-    {
-        while let Some(entry) = self.data_iter.next()
-        {
-            if (entry.freeze_flags & self.freeze_flags).is_empty()
-            {
-                return Some((entry.entity.clone(), &entry.component));
-            }
-        }
-
-        None
-    }
-}
-
-
-pub struct ComponentFilterIterMut<'a, C : Component + 'static>
-{
-    data_iter : std::slice::IterMut<'a, ComponentListEntry<C>>,
-    freeze_flags : FreezeFlags
-}
-
-impl<'a, C : Component + 'static> ComponentFilterIterMut<'a, C>
-{
-    fn new(store : &mut ComponentStore<C>, freeze_flags : FreezeFlags) -> ComponentFilterIterMut<C>
-    {
-        ComponentFilterIterMut::<C>
-        {
-            data_iter : store.component_list.iter_mut(),
-            freeze_flags
-        }
-    }
-}
-
-impl<'a, C : Component + 'static> Iterator for ComponentFilterIterMut<'a, C>
-{
-    type Item = (Entity, &'a mut C);
-
-    fn next(&mut self) -> Option<(Entity, &'a mut C)>
-    {
-        while let Some(entry) = self.data_iter.next()
-        {
-            if (entry.freeze_flags & self.freeze_flags).is_empty()
-            {
-                return Some((entry.entity.clone(), &mut entry.component));
-            }
-        }
-
-        None
-    }
-}
-
-
-////////////////////////////////////////////////////////////
-
-
 struct ComponentControlData
 {
     index_to_component_id : Vec<TypeId>,
@@ -1572,6 +1388,7 @@ impl ComponentControl
 trait DeferredComponentListBase : Downcast
 {
     fn add_to_scene(&mut self, scene_components : &mut SceneComponents);
+    fn priority(&self) -> i32;
 }
 impl_downcast!(DeferredComponentListBase);
 
@@ -1609,6 +1426,11 @@ impl<C : Component + 'static> DeferredComponentListBase for DeferredComponentLis
 
         scene_components.add_multiple_components(list);
     }
+
+    fn priority(&self) -> i32
+    {
+        C::add_priority()
+    }
 }
 
 
@@ -1783,7 +1605,8 @@ impl<T : SceneType + 'static> SceneControl<T>
     /// Registers a set of callbacks after the current [ThinkerSystem::think()] finishes. These
     ///  callbacks will fire whenever a [Component] of this type is added or removed. The callbacks
     ///  are not guaranteed to be called in the exact order that the components were created in this
-    ///  frame, as multiple components of a given type may be batched together.
+    ///  frame, as multiple components of a given type may be batched together. The callbacks will
+    ///  not be called if the component is assigned to directly.
     pub fn register_callbacks_later<C : Component + 'static>(&mut self, callbacks : ComponentCallbacks)
     {
         self.deferred_callback_adds.push((TypeId::of::<C>(), callbacks));
@@ -2673,6 +2496,8 @@ impl<T : SceneType + 'static> Scene<T>
             let mut deferred_component_adds = DeferredComponentAdds::new();
             std::mem::swap(&mut deferred_component_adds, &mut self.control.spawn_control.deferred_adds);
 
+            deferred_component_adds.lists.sort_by(|a, b| { a.priority().cmp(&b.priority()) });
+
             for list in &mut deferred_component_adds.lists
             {
                 list.add_to_scene(&mut self.components);
