@@ -1,7 +1,7 @@
 use crate::datapack::{DataId, DataMultistore};
 use crate::gameloop::{GameInfo, GameControl, GameControlOptions};
 use crate::renderer::{DrawTransform, DrawBackgroundColorInfo};
-use crate::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, DrawerSystem, ThinkerArgs, DrawerArgs, FreezeFlags};
+use crate::scene::{BaseScene, SimpleScene, NullSceneType, Entity, Component, ThinkerSystem, DrawerSystem, ThinkerArgs, DrawerArgs, FreezeFlags, ComponentRef, ComponentRefMut};
 use rand::seq::SliceRandom;
 
 #[derive(Debug)]
@@ -15,7 +15,30 @@ impl FooComponent
     }
 }
 
-impl Component for FooComponent {}
+impl Component for FooComponent
+{
+    fn maintain_ordering() -> bool where Self : Sized { false }
+}
+
+impl<'a> ComponentRef<'a> for FooComponent
+{
+    type StoreRef = &'a FooComponent;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for FooComponent
+{
+    type StoreRefMut = &'a mut FooComponent;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
 
 #[derive(Debug)]
 struct BarComponent { runs : u32 }
@@ -28,7 +51,30 @@ impl BarComponent
     }
 }
 
-impl Component for BarComponent {}
+impl Component for BarComponent
+{
+    fn maintain_ordering() -> bool where Self : Sized { false }
+}
+
+impl<'a> ComponentRef<'a> for BarComponent
+{
+    type StoreRef = &'a BarComponent;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for BarComponent
+{
+    type StoreRefMut = &'a mut BarComponent;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
 
 #[derive(Debug)]
 struct BazComponent { runs : u32 }
@@ -41,7 +87,30 @@ impl BazComponent
     }
 }
 
-impl Component for BazComponent {}
+impl Component for BazComponent
+{
+    fn maintain_ordering() -> bool where Self : Sized { false }
+}
+
+impl<'a> ComponentRef<'a> for BazComponent
+{
+    type StoreRef = &'a BazComponent;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for BazComponent
+{
+    type StoreRefMut = &'a mut BazComponent;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
 
 #[derive(Debug)]
 struct FooComponentOrdered { runs : u32 }
@@ -59,6 +128,26 @@ impl Component for FooComponentOrdered
     fn maintain_ordering() -> bool where Self : Sized { true }
 }
 
+impl<'a> ComponentRef<'a> for FooComponentOrdered
+{
+    type StoreRef = &'a FooComponentOrdered;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for FooComponentOrdered
+{
+    type StoreRefMut = &'a mut FooComponentOrdered;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 #[derive(Debug)]
 struct BarComponentOrdered { runs : u32 }
 
@@ -75,6 +164,26 @@ impl Component for BarComponentOrdered
     fn maintain_ordering() -> bool where Self : Sized { true }
 }
 
+impl<'a> ComponentRef<'a> for BarComponentOrdered
+{
+    type StoreRef = &'a BarComponentOrdered;
+
+    fn store_get(&'a self) -> Self::StoreRef
+    {
+        self
+    }
+}
+
+impl<'a> ComponentRefMut<'a> for BarComponentOrdered
+{
+    type StoreRefMut = &'a mut BarComponentOrdered;
+
+    fn store_get_mut(&'a mut self) -> Self::StoreRefMut
+    {
+        self
+    }
+}
+
 #[derive(Debug)]
 struct FooSystem {}
 
